#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Nov  2 18:16:30 2018

@author: Stefano Toniolo
"""

import numpy as np

class Acquisition(object):
    
    #def __init__(self, model, grad):
    #    self.model = model
    #    self.grad = grad

    def __init__(self, grad):
        self.grad = grad
        self.model = None
        
    def get_acquisition(self, x):
        x_acq = self._compute_acquisition(x)
        x_mask = self._mask_constraints(x)
        x_acq[x_mask == 0] = -1
        return -x_acq
    
    def get_acquisition_grad(self, x):
        x_acq_grad = self._compute_acquisition_grad(x)
        return -x_acq_grad * self._mask_constraints(x)
    
    def grad_support(self):
        return self.grad
    
    def set_constraints(self, constraints):
        self.constraints = constraints
    
    def set_model(self, model):
        self.model = model
        
    def _mask_constraints(self, x):
        x = np.atleast_2d(x)
        x_mask = np.ones((x.shape[0], 1))
        if self.constraints != None:
            for c in self.constraints:
                if c['type'] == 'ineq':
                    yvals = c['fun'](x)
                    mask = (yvals < 0) * 1
                    x_mask *= mask.reshape((x.shape[0], 1))
        return x_mask
    
    def _compute_acquisition(self, x):
        raise NotImplementedError("Under development")
        
    def _compute_acquisition_grad(self, x):
        raise NotImplementedError("Under development")