#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Nov  2 18:23:10 2018

@author: Stefano Toniolo
"""

from .acquisition import Acquisition

import numpy as np
from scipy.special import erfc
from scipy.stats import norm

class EI(Acquisition):
    
    #def __init__(self, model, jitter=0.0):
    #    super(EI, self).__init__(model, True)
    #    self.jitter = jitter
    
    def __init__(self, jitter=0.0):
        super(EI, self).__init__(True)
        self.jitter = jitter

    def _compute_acquisition(self, X):
        dim = self.model.model.X.shape[1]
        m, v = self.model.predict(X.reshape(-1,dim))
        v = np.clip(v, 1e-10, np.inf)
        s = np.sqrt(v)
        
        #fmax = np.max(self.model.model.Y)
        fmin = np.min(self.model.model.Y)
        
        if isinstance(s, np.ndarray):
            s[s<1e-10] = 1e-10
        elif s< 1e-10:
            s = 1e-10
            
        #u = (m - fmax - self.jitter)/s
        u = (fmin - m - self.jitter)/s
        phi = np.exp(-0.5 * u**2) / np.sqrt(2*np.pi)
        Phi = 0.5 * erfc(-u / np.sqrt(2))
        ei = s * (u * Phi + phi)
        #ei[s == 0.0] = 0.0
        
        return ei
        
    def _compute_acquisition_grad(self, X):
        dim = self.model.model.X.shape[1]
        m, v = self.model.predict(X.reshape(-1,dim))
        v = np.clip(v, 1e-10, np.inf)
        s = np.sqrt(v)
        
        if X.ndim==1: X = X[None,:]
        dmdx, dvdx = self.model.predictive_gradients(X.reshape(-1,dim))
        dmdx = dmdx[:,:,0]
        dsdx = dvdx / (2*np.sqrt(v))
        
        #fmax = np.max(self.model.model.Y)
        fmin = np.min(self.model.model.Y)
        
        if isinstance(s, np.ndarray):
            s[s<1e-10] = 1e-10
        elif s< 1e-10:
            s = 1e-10
            
        #u = (m - fmax - self.jitter)/s
        u = (fmin - m - self.jitter)/s
        phi = np.exp(-0.5 * u**2) / np.sqrt(2*np.pi)
        Phi = 0.5 * erfc(-u / np.sqrt(2))
        df_ei = dsdx * phi - Phi * dmdx
        #df_ei[s == 0.0] = 1.0

        return df_ei
