#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Dec 22 16:12:03 2018

@author: Stefano Toniolo
"""

from .acquisition import Acquisition
import numpy as np
class EIMC(Acquisition):
    
    #def __init__(self, model, g, N=1000):
    #    super(EIMC, self).__init__(model, False)
    #    self.g = g
    #    self.N = N
    
    def __init__(self, g, N=999):
        super(EIMC, self).__init__(False)
        self.g = g
        self.N = N
        
    def _improvement(self, gmin, X, y):
        #return np.maximum(0, self.g(y) - gmax)
        return np.maximum(0, gmin - self.g(X, y))
    
    def _compute_acquisition(self, X):
        dim = self.model[0].model.X.shape[1]
        X=X.reshape(-1,dim)
        z=[]
        for model in self.model:
            m, v = model.predict(X.reshape(-1,dim))
            # generate random numbers from Gaussian with mean μ and variance σ^2
            z.append(m + np.sqrt(v) * np.random.randn(self.N))
        
        # means of the predict of the models
        means=np.empty((self.all_X.shape[0], 0))
        for model in self.model:
            means=np.column_stack((means, model.predict(self.all_X)[0]))
        G=self.g(self.all_X, means)
            
        #gmax = np.max(G)
        gmin = np.min(G)
        
        result=np.empty((len(X), 0))
        for col in range(self.N):
            columns=np.empty((len(X), 0))
            for matrix in z:
                columns=np.column_stack((columns, matrix[:,col]))
            result=np.column_stack((result, self._improvement(gmin, X, columns).reshape(-1,1)))
        # expectation of improvement
        return np.mean(result, axis=1).reshape(-1,1)

    def set_all_X(self, all_X):
        self.all_X=all_X