#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Nov  3 16:24:40 2018

@author: Stefano Toniolo
"""

from .acquisition import Acquisition

import numpy as np

class LCB(Acquisition):
    
    def __init__(self, kappa=2.0):
        super(LCB, self).__init__(True)
        self.kappa = kappa
        
    def _compute_acquisition(self, x):
        dim = self.model.model.X.shape[1]
        m, v = self.model.predict(x.reshape(-1,dim))
        s = np.sqrt(v)
        lcb = -m + self.kappa * s
        return lcb
    
    def _compute_acquisition_grad(self, x):
        dim = self.model.model.X.shape[1]
        m, v = self.model.predict(x.reshape(-1,dim))
        
        if x.ndim==1: x = x[None,:]
        dmdx, dvdx = self.model.predictive_gradients(x.reshape(-1,dim))
        dmdx = dmdx[:,:,0]
        dsdx = dvdx / (2*np.sqrt(v))
        df_lcb = -dmdx + self.kappa * dsdx
        return df_lcb