# -*- coding: utf-8 -*-

import numpy as np


class EvaluatorBase(object):
    """
    Base class for the evaluator of the function. This class handles both sequential and batch evaluators.
    """

    def __init__(self, acquisition, batch_size, domain, 
                 normalize_X, samples_generator_type, 
                 one_hot_permutations, acquisition_optimizer):
        self.acquisition = acquisition
        self.batch_size = batch_size
        self.num_anchor = 5*batch_size
        self.domain=domain
        self.normalize_X=normalize_X
        self.samples_generator_type=samples_generator_type
        self.one_hot_permutations=one_hot_permutations
        self.acquisition_optimizer=acquisition_optimizer
        
    def compute_batch(self, duplicate_manager=None):
        # Do not care about having duplicates suggested
        if not duplicate_manager:
            return self.compute_batch_without_duplicate_logic()
        else:
            raise Exception("Can't handle duplicate Xs")
            
    # Bruteforce wrapper functions
    def _bruteforce_acqu(self, anchor_point):
        G = []
        for perm in self.one_hot_permutations:
            G.append(self.acquisition.get_acquisition(np.hstack((anchor_point, perm))))
        return max(G)
    
    def _bruteforce_acqu_grad(self, anchor_point):
        G = []
        for perm in self.one_hot_permutations:
            G.append(self.acquisition.get_acquisition_grad(np.hstack((anchor_point, perm))))
        return max(G)
    ##
    
    def _get_rounded(self, x):
        x_round = np.zeros_like(x)
        idx = 0
        for fvar in self.domain:
            dim = fvar.get_dim()
            x_round[:,idx:idx+dim] = fvar.get_rounded(x[:,idx:idx+dim], self.normalize_X)
            idx = idx + dim
        return x_round