# -*- coding: utf-8 -*-
from .base import EvaluatorBase
import sys
if "..." not in sys.path:
    sys.path.append("...")
from points_generator.anchor_points_generator import ObjectiveAnchorPointsGenerator
import numpy as np

class BatchSequential(EvaluatorBase):
    """
    This class handles both single X_next and multiple X_next with same logic
    (objective is acquisition function)
    """
    def __init__(self, acquisition, batch_size, domain, normalize_X, 
                 samples_generator_type, one_hot_permutations, acquisition_optimizer):

        super(BatchSequential, self).__init__(acquisition, batch_size, domain, normalize_X, samples_generator_type, one_hot_permutations, acquisition_optimizer)


    def get_anchor_points(self):
        anchor_points_generator = ObjectiveAnchorPointsGenerator(self.domain, self.acquisition.get_acquisition, self.normalize_X)
        return anchor_points_generator.get(num_anchor=self.num_anchor)

    def optimize_anchor_point(self, x0):
        if self.samples_generator_type == 'bruteforce':
            x0=x0[:next(i for i, var in enumerate(self.domain) if var.get_type()=='categorical')]
            x, y = self.acquisition_optimizer.optimize(x0, f=self._bruteforce_acqu)
            # search for optimal cat
            G = []
            for perm in self.one_hot_permutations:
                G.append(self.acquisition.get_acquisition(np.hstack((x[0], perm))))
            x = np.hstack((x, np.atleast_2d(self.one_hot_permutations[np.argmax(G),:])))
        else:
            if self.acquisition.grad_support():
                x, y = self.acquisition_optimizer.optimize(
                        x0,
                        f=self.acquisition.get_acquisition, 
                        df=self.acquisition.get_acquisition_grad)
            else:
                x, y = self.acquisition_optimizer.optimize(x0,
                                                           f=self.acquisition.get_acquisition)
        return self._get_rounded(x)
    
    def compute_batch_without_duplicate_logic(self):
        anchor_points = self.get_anchor_points()
        if self.batch_size==1:
            optimized_Xs=np.vstack([self.optimize_anchor_point(a) for a in anchor_points])
            optimized_Ys=self.acquisition.get_acquisition(optimized_Xs)
            return optimized_Xs[np.argmin(optimized_Ys)].reshape(1,-1)
        return np.vstack([self.optimize_anchor_point(a) for a, _ in zip(anchor_points, range(self.batch_size))])