# -*- coding: utf-8 -*-
from .base import EvaluatorBase
import sys
if "..." not in sys.path:
    sys.path.append("...")
from points_generator.anchor_points_generator import ThompsonSamplingAnchorPointsGenerator
import numpy as np

class ThompsonBatch(EvaluatorBase):
    """
    Class for a Thompson batch method. Elements are selected iteratively using the current acquistion function but exploring the models
    by using Thompson sampling
    :param acquisition: acquisition function to be used to compute the batch.
    :param batch size: the number of elements in the batch.
    """
    def __init__(self, acquisition, batch_size, domain, normalize_X, 
                 samples_generator_type, one_hot_permutations, acquisition_optimizer):

        super(ThompsonBatch, self).__init__(acquisition, batch_size, domain, normalize_X, samples_generator_type, one_hot_permutations, acquisition_optimizer)
        #self.model = self.acquisition.model

    def get_anchor_points(self):
        #anchor_points_generator = ThompsonSamplingAnchorPointsGenerator(self.domain, self.model, self.normalize_X)
        anchor_points_generator = ThompsonSamplingAnchorPointsGenerator(self.acquisition, self.model, self.normalize_X)
        return anchor_points_generator.get(num_anchor=self.num_anchor)

    def optimize_anchor_point(self, x0):


        if self.samples_generator_type == 'bruteforce':
            x, y = self.acquisition_optimizer.optimize(x0, f=self._bruteforce_acqu)
            # search for optimal cat
            G = []
            for perm in self.one_hot_permutations:
                G.append(self.acquisition.get_acquisition(np.hstack((x[0], perm))))
                x = np.hstack((x, np.atleast_2d(self.one_hot_permutations[np.argmax(G),:])))
        else:
            if self.acquisition.grad_support():
                x, y = self.acquisition_optimizer.optimize(x0,
                                                           f=self.acquisition.get_acquisition, 
                                                           df=self.acquisition.get_acquisition_grad)
            else:
                x, y = self.acquisition_optimizer.optimize(x0,
                                                           f=self.acquisition.get_acquisition)
        return self._get_rounded(x)
    
    def compute_batch_without_duplicate_logic(self):
        anchor_points = self.get_anchor_points()
        return np.vstack([self.optimize_anchor_point(a) for a, _ in zip(anchor_points, range(self.batch_size))])