# =============================================================================
# 
# # -*- coding: utf-8 -*-
# import sys
# sys.path.append("..")
# import core.BO as BO
# import numpy as np
# 
# def f(X):
#     n = X.shape[0]
#     fval = (X * np.sin(X) + 0.1 * X).sum(axis=1) 
#     return -fval.reshape(n,1)
# 
# fvars = [{'name':'var1', 'type':'continuous', 'domain': (-5.0,5.0)},
#          {'name':'var2', 'type':'continuous', 'domain': (-5.0,5.0)},
#          {'name':'var3', 'type':'continuous', 'domain':(-5.0,5.0)},
#          {'name':'var4', 'type':'discrete','domain':(3,8,9)},
#          {'name':'var5', 'type':'discrete', 'domain':(3,8,9)},
#          {'name':'var6', 'type':'categorical', 'domain':(0,1,3)},
#          {'name':'var7', 'type':'continuous','domain':(-1.0,2.0)}]
# 
# init_samples=5
# kernel = 'Matern32'
# my_bo=BO.BayesianOptimization(fvars, f, normalize_X=True, init_n_samples=init_samples, 
#                               samples_generator='bruteforce', ARD=True, kernel=kernel)
# my_bo.run_optimization(max_iter=20)
# print(np.min(my_bo.Y))
# =============================================================================

#%%

# -*- coding: utf-8 -*-
#import sys
#sys.path.append("..")
import core.BO as BO
import numpy as np
import GPy.kern as k

def f(X, noise=0.0):
    fval = -np.sin(3*X) - X**2 + 0.7*X
    fval_noise = noise * np.random.randn(*X.shape)
    return -fval

def ackley_function(X):
    fval=-20*np.exp(-0.2*np.sqrt(0.5*(X[:,0]**2+X[:,1]**2)))-np.exp(0.5*(np.cos(2*np.pi*X[:,0])+np.cos(2*np.pi*X[:,1])))+np.e+20
    return -fval.reshape(-1, 1)

def six_humps(X):
    ''' f(x,y) = (4 - 2.1x^2 + x^4 / 3)x^2 + xy + (-4 + 4x^2)x^2 '''
    x2 = np.power(X[:,0],2)
    x4 = np.power(X[:,0],4)
    y2 = np.power(X[:,1],2)  
    fval = (4.0 - 2.1 * x2 + (x4 / 3.0)) * x2 + X[:,0]*X[:,1] + (-4.0 + 4.0 * y2) * y2
    return fval.reshape(-1,1)

def rastrigin(X):
    #-5.12<x<5.12
    part=0
    for i in range(0,X.shape[1]):
        part+=X[:,i]**2-10*np.cos(2*np.pi*X[:,i])
    fval=10*X.shape[1]+part
    return -fval.reshape(-1,1)

def tripod(X):
    def p(X):
        return np.heaviside(X,1)
    fval=p(X[:,1])*(1+p(X[:,0]))+np.abs(X[:,0]+50*p(X[:,1])*(1-2*p(X[:,0])))+\
        np.abs(X[:,1]+50*(1-2*p(X[:,1])))
    return -fval.reshape(-1,1)

def sphere(X):
    fval=0
    for i in range(0,X.shape[1]):
        fval+=X[:,i]**2
    return -fval.reshape(-1,1)

def six_humps_cat(X):
    ''' 
    Six humps custom with categoricals
     - Global min: Y = -1.0316+0.25+0.4=-0.3816
                   X = [[-3,3],[-2,2],[1,2,...,N],[1,2,...,N]]
    '''
    x2 = np.power(X[:,0],2)
    x4 = np.power(X[:,0],4)
    y2 = np.power(X[:,1],2)  
    fval = (4.0 - 2.1 * x2 + (x4 / 3.0)) * x2 + X[:,0]*X[:,1] + (-4.0 + 4.0 * y2) * y2+X[:,2]*0.25+X[:,3]*0.4
    return fval.reshape(-1,1) 

def fy(x, n=10):
    return x+np.random.randn(n)*0.1 #this is the noisy output 

fvars = [{'name':'var1', 'type':'continuous', 'domain': (-3,3)},
                     {'name':'var2', 'type':'continuous', 'domain': (-2,2)},
                     {'name':'var3', 'type':'categorical', 'domain': (1,2,3)},
                     {'name':'var4', 'type':'categorical', 'domain': (1,2,3)}]

# Define kernel
kernel=k.Matern52(input_dim=8)

# Define initial samples
init_samples=3
# Create the BayesianOptimization object
my_bo=BO.BayesianOptimization(fvars, [six_humps_cat], normalize_X=False, normalize_Y=True,
                             init_n_samples=init_samples, samples_generator='bruteforce', 
                             ARD=True, kernel=kernel)
    
my_bo.run_optimization(max_iter=20,plot=False)
print("Min found: {0}".format(np.min(my_bo.Y[0])))
print("x: {}".format(my_bo.X[0][np.argmin(my_bo.Y[0])])) 
#print("x:",my_bo.X[np.argmax(my_bo.Y)]," y:",-np.max(my_bo.Y))
