#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  1 14:00:50 2019

@author: Stefano Toniolo
"""

import numpy as np
import core.BO as BO
import GPy.kern as k

def six_humps_cat(X):
    ''' 
    Six humps custom with categoricals
     - Global min: Y = -1.0316+0.25+0.4=-0.3816
                   X = [[-3,3],[-2,2],[1,2,...,N],[1,2,...,N]]
    '''
    x2 = np.power(X[:,0],2)
    x4 = np.power(X[:,0],4)
    y2 = np.power(X[:,1],2)  
    fval = (4.0 - 2.1 * x2 + (x4 / 3.0)) * x2 + X[:,0]*X[:,1] + (-4.0 + 4.0 * y2) * y2+X[:,2]*0.25
    return fval.reshape(-1,1) 

fvarsX = [{'name':'x1', 'type':'continuous', 'domain': (-3,3)},
          {'name':'x2', 'type':'continuous', 'domain': (-2,2)},
          {'name':'x3', 'type':'discrete', 'domain': np.arange(-1,300.2,20)}]

kernel = k.Matern52(input_dim=3)
run_iter = 50

bo = BO.BayesianOptimization(
                fvarsX, [six_humps_cat], kernel=kernel, normalize_Y=True,
                acquisition_type='EI', acquisition_optimizer_type='lbfgsb',
                samples_generator='random', evaluator_type='sequential_batch')

bo.run_optimization(run_iter)

print("x:",bo.X[np.argmin(bo.Y)]," y:",np.min(bo.Y))

# Check
fault = False
for n in bo.X[:,2]:
    if n not in np.arange(-1,300.2,20):
        print("Fault at value {0}".format(n))
        fault = True
if not fault:
    print("Check ok")