#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb  2 15:09:52 2019

@author: Stefano Toniolo
"""

import core.BO as BO
import numpy as np
import GPy.kern as k
import matplotlib.pyplot as plt

def franke2d(xx):
    #print(xx)
    x1 = xx[:,0]
    x2 = xx[:,1]

    term1 = 0.75 * np.exp(-(9*x1-2)**2/4 - (9*x2-2)**2/4);
    term2 = 0.75 * np.exp(-(9*x1+1)**2/49 - (9*x2+1)/10);
    term3 = 0.5 * np.exp(-(9*x1-7)**2/4 - (9*x2-3)**2/4);
    term4 = -0.2 * np.exp(-(9*x1-4)**2 - (9*x2-7)**2);
    
    fval=term1 + term2 + term3 + term4

    return  -fval.reshape(-1,1)

x = np.linspace(0,1,50)
y = np.linspace(0,1,50)
X, Y = np.meshgrid(x,y)
Z = np.zeros((50,50))

for i in range(50):
    for j in range(50):
        Z[i,j] = franke2d(np.array([[X[i,j],Y[i,j]]]))

plt.pcolor(X, Y, Z)
plt.show()

def g(X, Y):
    return Y[:,0]+Y[:,1]

def sub_objective1(X):
    print(X)
    return X[:,0]**2

def sub_objective2(X):
    return X[:,1]**2


fvarsX = [{'name':'x1', 'type':'discrete', 'domain': np.arange(-1,300,10)},
          {'name':'x2', 'type':'continuous', 'domain': (-1,1)}]

fvarsY = [{'name':'y1', 'type':'continuous', 'domain': (-100,100)},
          {'name':'y2', 'type':'continuous', 'domain': (-100,100)}]

kernel = k.Matern52(input_dim=2)# + k.Bias(input_dim=1)
sub_objectives=[sub_objective1, sub_objective2]

# Define initial samples
init_samples=3

'''
If you wanna use a non-EIMC acquisition or if you have a single output,
just put a single sub_objective in the list and make g() simply return Y
'''

# Create the BayesianOptimization object
my_bo=BO.BayesianOptimization(fvarsX, sub_objectives, fvarsY, g, normalize_Y=True, normalize_X=True,
                              acquisition_type='EIMC', acquisition_optimizer_type='lbfgsb',
                              init_n_samples=init_samples, samples_generator='random', 
                              ARD=True, kernel=kernel)

my_bo.run_optimization(max_iter=3,plot=False)

print("best y:",np.min(g(None, my_bo.Y)))