# -*- coding: utf-8 -*-

import core.BO as BO
import numpy as np
import GPy.kern as k

def f(X, noise=0.0):
    fval = (6*X-2)**2*np.sin(12*X-4)
    return fval

def fsin(X):
    return np.sin(X)

def six_humps(X, noise=0.0):
    ''' f(x,y) = (4 - 2.1x^2 + x^4 / 3)x^2 + xy + (-4 + 4x^2)x^2 '''
    x2 = np.power(X[:,0],2)
    x4 = np.power(X[:,0],4)
    y2 = np.power(X[:,1],2)  
    fval = (4.0 - 2.1 * x2 + (x4 / 3.0)) * x2 + X[:,0]*X[:,1] + (-4.0 + 4.0 * y2) * y2
    noise = np.random.normal(0,noise)
    fval = fval + noise
    return fval.reshape(-1,1)



def g(X, Y):
    return Y[:,0]+Y[:,1]

def sub_objective1(X):
    return X[:,0]**2+X[:,1]**2

def sub_objective2(X):
    return X[:,0]**2


fvarsX = [{'name':'x1', 'type':'continuous', 'domain': (-1,1)},
    {'name':'x2', 'type':'continuous', 'domain': (-1,1)}]
fvarsY = [{'name':'y1', 'type':'continuous', 'domain': (-100,100)},
           {'name':'y2', 'type':'continuous', 'domain': (-100,100)}]
kernel = k.Matern52(input_dim=2)# + k.Bias(input_dim=1)
sub_objectives=[sub_objective1, sub_objective2]

# Define initial samples
init_samples=3

'''
If you wanna use a non-EIMC acquisition or if you have a single output,
just put a single sub_objective in the list and make g() simply return Y
'''


# Create the BayesianOptimization object
my_bo=BO.BayesianOptimization(fvarsX, sub_objectives, fvarsY, g=g, normalize_Y=True, normalize_X=True,
                              acquisition_type='EIMC', acquisition_optimizer_type='lbfgsb',
                              init_n_samples=init_samples, samples_generator='random', 
                              ARD=True, kernel=kernel, save_experiment=False)

my_bo.run_optimization(max_iter=5,plot=False)

my_bo.compute_results()
print(my_bo.X)

#%% gpyopt comparison

# =============================================================================
# import GPy
# import GPyOpt
# 
# # create the object function
# f_true = GPyOpt.objective_examples.experiments2d.sixhumpcamel()
# f_sim = GPyOpt.objective_examples.experiments2d.sixhumpcamel(sd = 0.1)
# 
# bounds =[{'name': 'var_1', 'type': 'continuous', 'domain': f_true.bounds[0]},
#          {'name': 'var_2', 'type': 'continuous', 'domain': f_true.bounds[1]}]
# f_true.plot()
# 
# 
# 
# # Creates three identical objects that we will later use to compare the optimization strategies 
# myBopt2D = GPyOpt.methods.BayesianOptimization(f_sim.f,
#                                               domain=bounds,
#                                               model_type = 'GP',
#                                               acquisition_type='EI',  
#                                               normalize_Y = True)    
# 
# 
# 
# # runs the optimization for the three methods
# max_iter = 10  # maximum time 40 iterations
# max_time = 60  # maximum time 60 seconds
# 
# myBopt2D.run_optimization(max_iter,max_time,verbosity=False)            
# myBopt2D.plot_acquisition() 
# 
# myBopt2D.Y_best
# 
# =============================================================================
