# -*- coding: utf-8 -*-
import numpy as np
import GPy

class GPModel(object):
    def __init__(self, kernel=None, model_optimizer_type='bfgs', optimize_restarts=5, max_iters=1000, ARD=False, verbose=False, normalize_Y=None):
        self.kernel=kernel
        self.optimize_restarts=optimize_restarts
        self.model_optimizer_type=model_optimizer_type
        self.ARD=ARD
        self.model=None
        self.max_iters=max_iters
        self.verbose=verbose
        self.normalize_Y=None if (normalize_Y==False or normalize_Y==None) else True
        
        
    def _create_model(self, X, Y):
        """
        Creates the model given some input data X and Y.
        """

        # --- define kernel
        self.input_dim = X.shape[1]
        if self.kernel is None:
            self.kernel = GPy.kern.Matern52(self.input_dim, ARD=self.ARD)
        #else:
        #    self.kernel=getattr(GPy.kern, self.kernel)(self.input_dim, ARD=self.ARD)

        self.model = GPy.models.GPRegression(X, Y, kernel=self.kernel, normalizer=self.normalize_Y)

    def updateModel(self, X, Y):
        """
        Updates the model with new observations.
        """
        if self.model is None:
            self._create_model(X, Y)
        else:
            self.model.set_XY(X, Y)
            
        if self.max_iters>0:
            if self.optimize_restarts==1:
                self.model.optimize(optimizer=self.model_optimizer_type, max_iters = self.max_iters, messages=False, ipython_notebook=False)
            else:
                self.model.optimize_restarts(num_restarts=self.optimize_restarts, optimizer=self.model_optimizer_type, max_iters = self.max_iters, verbose=self.verbose)
                
    def predict(self, X):
        m, v = self.model.predict(X)
        v = np.clip(v, 1e-10, np.inf)
        return m, v
    
    def predictive_gradients(self, X):
        return self.model.predictive_gradients(X)