#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jan 25 23:33:07 2019

@author: Stefano Toniolo
"""

import numpy as np
import GPy.kern as k
import core.BO as BO

def six_humps_cat(X):
    ''' 
    Six humps custom with categoricals
     - Global min: Y = -1.0316+0.25+0.4=-0.3816
                   X = [[-3,3],[-2,2],[1,2,...,N],[1,2,...,N]]
    '''
    x2 = np.power(X[:,0],2)
    x4 = np.power(X[:,0],4)
    y2 = np.power(X[:,1],2)  
    fval = (4.0 - 2.1 * x2 + (x4 / 3.0)) * x2 + X[:,0]*X[:,1] + (-4.0 + 4.0 * y2) * y2+X[:,2]*0.25+X[:,3]*0.4
    return fval.reshape(-1,1) 

fvar = [{'name':'var1', 'type':'continuous', 'domain': (-3,3)},
             {'name':'var2', 'type':'continuous', 'domain': (-2,2)},
             {'name':'var3', 'type':'categorical', 'domain': (1,2,3)},
             {'name':'var4', 'type':'categorical', 'domain': (1,2,3)}]
kernel = k.Matern52(input_dim=8)
epsilon = 0.1
run_iter = 20
global_miny = -1.0316+0.25+0.4

bo = BO.BayesianOptimization(
        fvar, [six_humps_cat], kernel=kernel, normalize_Y=True,
        acquisition_type='EI', acquisition_optimizer_type='lbfgsb',
        samples_generator='bruteforce', evaluator_type='sequential_batch')
bo.run_optimization(run_iter)

miny = np.min(bo.Y[0])
print("Min found: {0}".format(np.min(bo.Y[0])))
print("x: {}".format(bo.X[0][np.argmin(bo.Y[0]),:]))       

#%%

import GPy
import GPyOpt

fvar = [{'name':'var1', 'type':'continuous', 'domain': (-3,3)},
        {'name':'var2', 'type':'continuous', 'domain': (-2,2)},
        {'name':'var3', 'type':'categorical', 'domain': (1,2,3)},
        {'name':'var4', 'type':'categorical', 'domain': (1,2,3)}]



# Creates three identical objects that we will later use to compare the optimization strategies 
myBopt2D = GPyOpt.methods.BayesianOptimization(six_humps_cat,
                                               domain=fvar,
                                               model_type = 'GP',
                                               acquisition_type='EI',  
                                               normalize_Y = True)    


# runs the optimization for the three methods
max_iter = 40  # maximum time 40 iterations
max_time = 60  # maximum time 60 seconds

myBopt2D.run_optimization(max_iter,max_time,verbosity=False)            

myBopt2D.plot_acquisition() 
