# -*- coding: utf-8 -*-
from points_generator.random_points_generator import RandomPointsGenerator
import sys
sys.path.append("..")
from utils.math import normalize_X

import numpy as np

class AnchorPointsGenerator(object):

    def __init__(self, domain, num_samples, normalize_X):
        self.domain=domain
        self.num_samples = num_samples
        self.normalize_X=normalize_X

    def get(self, num_anchor=5):
        # num_anchor by default is batch_size*5
        
        generator=RandomPointsGenerator(self.domain)
        X = generator.generate_points(self.num_samples)
        if self.normalize_X:
            X=normalize_X(self.domain, X)
        scores = self.get_anchor_point_scores(X)
        
        # u sure u're filtering the bests?
        anchor_points = X[np.argsort(scores)[:min(len(scores),num_anchor)], :]

        return anchor_points


class ObjectiveAnchorPointsGenerator(AnchorPointsGenerator):

    def __init__(self, domain, acquisition, normalize_X, num_samples=1000):
        """
        Objective is usually the acquisition function
        """
        
        super(ObjectiveAnchorPointsGenerator, self).__init__(domain, num_samples, normalize_X)
        self.acquisition = acquisition

    def get_anchor_point_scores(self, X):
        return self.acquisition(X).flatten()
    
class ThompsonSamplingAnchorPointsGenerator(AnchorPointsGenerator):

    #def __init__(self, domain, model, normalize_X, num_samples=25000):
    def __init__(self, domain, acquisition, normalize_X, num_samples=25000):
        '''
        It selects the location using (marginal) Thompson sampling
        using the predictive distribution of a model
        '''
        super(ThompsonSamplingAnchorPointsGenerator, self).__init__(domain, num_samples, normalize_X)
        #self.model = model
        self.acquisition = acquisition

    def get_anchor_point_scores(self, X):
        #posterior_means, posterior_variance = self.model.predict(X)
        posterior_means, posterior_variance = self.acquisition.model.predict(X)
        posterior_stds=np.sqrt(posterior_variance)
        posterior_means=-posterior_means
        return np.array([np.random.normal(m, s) for m, s in zip(posterior_means, posterior_stds)]).flatten()