#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 14 21:26:37 2019

@author: Stefano Toniolo
"""

import unittest
import numpy as np
import GPy.kern as k
import core.BO as BO

def forrester(x):
    '''
    Forrester function
     - global min [0,1]: y = -6.02074
                         x = 0.757249
    '''
    return (6*x-2)**2*np.sin(12*x-4)  

class TestForrester(unittest.TestCase):
    
    def setUp(self):
        self.fvar = [{'name':'var1', 'type':'continuous', 'domain': (0,1)}]
        self.kernel = k.Matern52(input_dim=1)
        self.epsilon = 0.1
        self.global_miny = -6.02074
    
    def test_bo_ei(self):
        bo = BO.BayesianOptimization(
                self.fvar, [forrester], kernel=self.kernel, normalize_Y=True,
                acquisition_type='EI', acquisition_optimizer_type='lbfgsb',
                samples_generator='random', evaluator_type='sequential_batch')
        bo.run_optimization()
        
        miny = np.min(bo.Y)
        print("Min found: {0}".format(np.min(bo.Y)))
               
        self.assertTrue(self.global_miny - self.epsilon <= miny <= self.global_miny + self.epsilon,
                        'Min Y not found')
        
    def test_bo_eimc(self):
        bo = BO.BayesianOptimization(
                self.fvar, [forrester], kernel=self.kernel, normalize_Y=True,
                acquisition_type='EIMC', acquisition_optimizer_type='lbfgsb',
                samples_generator='random', evaluator_type='sequential_batch')
        bo.run_optimization()
        
        miny = np.min(bo.Y)
        print("Min found: {0}".format(np.min(bo.Y)))
        
        self.assertTrue(self.global_miny - self.epsilon <= miny <= self.global_miny + self.epsilon,
                        'Min Y not found')

if __name__ == '__main__':
    unittest.main()